const char* tutorial_en_jicfs4=\
"CategoryCode4,Category4\n\
1101,Seasoning\n\
1102,Edible_Oil\n\
1103,Spreads\n\
1104,Dairy_Product\n\
1105,Condiements\n\
1106,Soup\n\
1107,Frozen_Food\n\
1108,Canned_Goods\n\
1110,Powder\n\
1111,Homemaking_Material\n\
1112,Noodles\n\
1113,Bread\n\
1114,Grain\n\
1115,Processed_Meats\n\
1116,Fish_Cake\n\
1117,Salted_Pickles\n\
1118,Food_in_Water\n\
1119,Delicatessen\n\
1120,Dried_Agricultrual_Produce\n\
1121,Processed_Seafood\n\
1197,Miscellanous_Processed_Food\n\
1201,Seafood\n\
1202,Meat_Produce\n\
1203,Agricultural_Product\n\
1297,Miscellanous_Fresh_Produce\n\
1301,Confectionery\n\
1302,Dessert_Yogurt\n\
1303,Distinctive_Dry_Foods\n\
1304,Ice_Cream\n\
1397,Other_Confectionery\n\
1401,Flavored_Drinks\n\
1402,Juice\n\
1403,Soft_Drinks\n\
1404,Yogurt_Drinks\n\
1406,Alcoholic_Drinks\n\
1407,Merchandise_With_Alcohol\n\
1497,Miscellanous_Drinks\n\
1901,Baby_Food\n\
1902,Health_Food\n\
1903,Gift_Food\n\
1997,Miscellanous_Food\n\
1999,Unknown_Food\n\
2101,Oral_Hygienic_Supplies\n\
2102,Soaps\n\
2103,Toilet_Paper_Supplies\n\
2104,Medical_Supplies_Equipment\n\
2105,Childcare_Supplies\n\
2106,Clothing_Detergents\n\
2107,House_Cleaning_Detergents\n\
2108,Kitchen_Tableware_Detergents\n\
2109,Home_Envirnoment_Agents\n\
2110,Home_Cleaner_Wax\n\
2111,Insecticide\n\
2197,Miscellanous_Household_Products\n\
2199,Unknown_Household_Products\n\
2201,Vitamins\n\
2202,Painkillers\n\
2203,Digestion_Medicine\n\
2204,Skin_Medicine\n\
2205,Chinese_Medicine\n\
2297,Miscellanous_Medicine\n\
2299,Unknown_Medicine\n\
2301,Fundamental_Cosmetics\n\
2302,Makeup_Cosmetics\n\
2303,Fragrance\n\
2304,Hair_Care\n\
2305,Men_Cosmetics\n\
2306,Small_Makeup_Articles\n\
2397,Other_Cosmetics\n\
2399,Unknown_Cosmetics\n\
2401,Food_Packaging\n\
2402,Cleaning_Supplies\n\
2403,Laundry_Dryer_Supplies\n\
2404,Kitchen_Supplies\n\
2405,Bathroom_Supplies\n\
2406,Dining_Table_Supplies\n\
2407,Cooking_Supplies\n\
2408,Living_Supplies\n\
2497,Other_Household_Supplies\n\
2499,Unknown_Household_Supplies\n\
2501,Construction_Paint_Equipment\n\
2502,Construction_Paint_Supplies\n\
2503,Gas_Plumbing_Parts\n\
2504,Gardening_Tools\n\
2597,Miscellanous_DIY_Supplies\n\
2599,Unknown_DIY_Supplies\n\
2601,Pet_House\n\
2602,Pet_Medicine\n\
2603,Pet_Equipment\n\
2697,Miscellanous_Pet_Products\n\
2699,Unknown_Pet_Products\n\
2901,Daily_Gift\n\
2903,Film\n\
2904,Audio_Stereo\n\
2997,Miscellanous_Daily_Necessaries\n\
2999,Unknown_Daily_Necessaries\n\
3101,Information_Organizing_Supplies\n\
3102,File_Binder\n\
3103,Office_Supplies\n\
3104,Writing_Equipment\n\
3105,Paper_Products\n\
3106,Daily_Paper_Products\n\
3107,Education_Related_Supplies\n\
3108,Electronic_Stationary\n\
3109,Business_Use_Machine\n\
3110,Business_Machine\n\
3111,OA_Software\n\
3112,Design_Drafting_Supplies\n\
3113,Design_Picture_Materials\n\
3115,Office_Furniture\n\
3116,Portable_Stationary\n\
3117,Stationary_Office_Supplies_Gift\n\
3118,Electronic_Related_Stationary\n\
3197,General_Stationary_Office_Supplies\n\
3201,General_Game\n\
3202,Puzzle\n\
3203,Electronic_Game\n\
3204,Video_Game\n\
3205,Boy_Character_Figures\n\
3206,Stuff_Toy\n\
3207,Mini_Car\n\
3208,Train_Rails\n\
3209,Science_Control\n\
3210,Kisekae\n\
3211,Mamagoto\n\
3212,Girl_Hobby\n\
3213,Toy_Doll\n\
3214,Intellectual_Growth_Music_Toy\n\
3215,Nurture_Toy\n\
3216,Fancy\n\
3217,Sewing_Related\n\
3218,Plane_Model\n\
3219,Radio_Control\n\
3220,Rail_Model\n\
3221,Gun\n\
3222,Craft_Hobby\n\
3223,Hobby_Related_Supplies\n\
3224,Gift_Doll\n\
3225,Seasonal_Supplies\n\
3226,Fireworks\n\
3227,Little_Toy\n\
3228,Sport\n\
3229,Vehicle\n\
3297,Miscellanous\n\
3299,Unknown_Toy\n\
3397,General_Books\n\
3399,Unknown_Books\n\
3401,Pianos\n\
3402,Music_Organs\n\
3403,Wind_Instrument\n\
3404,String_Instrument\n\
3405,Percussion_Instrument\n\
3406,Electronic_Musical_Instrument\n\
3407,Japanese_Musical_Instrument\n\
3408,Musical_Instrument_Parts_Accessories\n\
3409,Recording\n\
3497,General_Musical_Instrument_Software\n\
3499,Unknown_Musical_Instrument_Software\n\
3599,Unknown_Sporting_Goods\n\
3601,Digital\n\
3602,Analog\n\
3603,Hybrid\n\
3604,System_Program\n\
3605,Application_Program\n\
3606,Wire_Communication_Device\n\
3697,General_Information_Equipment\n\
3699,Unknown_Information_Equipment\n\
3901,Match_Candle_Incense_Stick\n\
3902,Small_Needlework_Tools\n\
3903,Sign_Advertisement_Supplies\n\
3904,Religious_Supplies\n\
3997,General_Culture_Supplies\n\
3999,Unknown_Culture_Supplies\n\
4201,Cabinet\n\
4202,Closet\n\
4203,Shelf\n\
4204,Box\n\
4205,Stool\n\
4206,Desk\n\
4207,Table\n\
4208,Dresser\n\
4209,Stand\n\
4210,Chair\n\
4211,Bed\n\
4212,Matress\n\
4213,Safety_Box\n\
4214,Locker\n\
4215,Container_Stand\n\
4297,General_Furniture\n\
4299,Unknown_Furniture\n\
4301,Car_Accessories\n\
4303,Motor_Bike\n\
4305,Bicycle\n\
4397,General_Car_Accessories\n\
4399,Unknown_Car_Accessories\n\
4401,Clock\n\
4402,Time_Related_Products\n\
4405,Glasses\n\
4497,General_Clock_Products\n\
4499,Unknown_Clock_Products\n\
4501,Camera\n\
4502,Interchangeable_Lens\n\
4503,Optical_Equipment\n\
4504,Photography_Supplies\n\
4505,Film\n\
4597,Miscellanous_Photography_Supplies\n\
4599,Unknown_Photography_Supplies\n\
4601,Television_Related\n\
4603,Video_Related\n\
4605,VDP_Related\n\
4607,Stereo_Related\n\
4609,Stereo\n\
4611,Miscellanous_Audio\n\
4613,Radio_Related\n\
4615,Car_Related\n\
4617,Transceiver_Loudspeaker_Related\n\
4619,Magnetic_Tape_Related\n\
4621,Air_Conditioner_Equipment_Accessories\n\
4623,Fan\n\
4625,Central_Heater\n\
4627,Portable_Heater\n\
4629,Kitchen_Appliances\n\
4631,Range\n\
4633,Rotary_Cooking_Device\n\
4635,Electric_Cooking_Device\n\
4637,Cooking_Related_Device\n\
4639,Refrigeration_Appliances\n\
4641,Housekeeping_Supplies\n\
4643,Beauty_Care_Supplies\n\
4645,Health_Medical_Equipment\n\
4647,Illumination_Supplies\n\
4649,Business_School_Supplies\n\
4651,Clock_Camera\n\
4653,Musical_Instrument\n\
4655,OA_Machinery\n\
4657,Residence_Supplies\n\
4659,Carpenter_Gardening_Leisure_Supplies\n\
4661,Leisure_Home_Appliances\n\
4663,Consumables\n\
4697,Other_Appliances\n\
4699,Unknown_Appliances\n\
4901,Fine_Arts_Craftwork\n\
4902,Camera\n\
4903,Sewing_Machine\n\
4904,Fine_Metals\n\
4997,General_Durable_Consumer_Goods\n\
4999,Unknown_Durable_Consumer_Goods\n\
5101,Underwear_Socks\n\
5102,Child_Clothing\n\
5103,Mens_Clothing\n\
5104,Womens_Clothing\n\
5105,Sports_Wear\n\
5106,Drapery\n\
5107,Formal\n\
5108,Jeans\n\
5197,General_Clothing\n\
5199,Unknown_Clothing\n\
5301,Futon\n\
5302,Blanket\n\
5303,Sheets_Covers\n\
5304,Bed_Accessories\n\
5305,Foot_Warmer\n\
5306,Baby_Sleepwear\n\
5307,Bed_Supplies\n\
5308,Towels\n\
5309,Interior_Supplies\n\
5397,General_Bed_Supplies\n\
5399,Unknown_Bed_Supplies\n\
5401,Child_Clothing\n\
5402,Mens_Clothing\n\
5403,Womens_Clothing\n\
5404,Japanese_Dress\n\
5405,Handbags\n\
5406,Bag\n\
5407,Umbrella_Stick\n\
5408,Japanese_Fan\n\
5409,Ornament\n\
5497,General_Apparel\n\
5501,Baby\n\
5502,Teens_Kids(Leather)\n\
5503,Senior(Artifical_Leather)\n\
5504,Mens(Leather)\n\
5505,Mens(Artifical_Leather)\n\
5506,Mens(Other)\n\
5507,Womens(Leather)\n\
5508,Womens(Artifical_Leather)\n\
5509,Womens(Other)\n\
5510,Miscellanous_Shoes\n\
5599,Unknown_Shoes\n\
5601,Baseball_Softball\n\
5602,Racquets_Accessories\n\
5603,Field_Truck_Equipment\n\
5604,Tennis_Tabletennis_Badminton_Equipment\n\
5605,Golfing_Equipment\n\
5606,Ski_Ice_Hockey_Ice_Skating_Equipment\n\
5607,Boxing_Martial_Arts\n\
5608,Gymnastics\n\
5609,Fishing_Diving_Swimming\n\
5697,General_Sports_Equipment\n\
5999,Unknown_Clothing_Appparel\n\
9901,Miscellanous_Imports\n\
9999,Unknown_Product\n";
