/* ////////// LICENSE INFO ////////////////////

 * Copyright (C) 2013 by NYSOL CORPORATION
 *
 * Unless you have received this program directly from NYSOL pursuant
 * to the terms of a commercial license agreement with NYSOL, then
 * this program is licensed to you under the terms of the GNU Affero General
 * Public License (AGPL) as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
 * for more details.

 ////////// LICENSE INFO ////////////////////*/
// =============================================================================
/// kguniqHelp.h : kguniq help
// =============================================================================
_titleL="レコードの単一化＆cat";
_docL="\
\n\
MUNICAT レコードの単一化＆cat\n\
\n\
値が重複した行を単一化してcat。\n\
\n\
書式\n\
\n\
muniq [k=] [K=]  [i=] [o=] [-assert_diffSize] [-assert_nullkey] [-nfn]\n\
[-nfno] [-x] [-q] [tmpPath=] [--help] [--helpl] [--version]\n\
\n\
パラメータ\n\
\n\
  k=   行を単一化する単位となる項目名リストを指定する。\n\
  K=   行を単一化する単位となる項目名リストを指定する。\n\
\n\
";

