/* ////////// LICENSE INFO ////////////////////

 * Copyright (C) 2013 by NYSOL CORPORATION
 *
 * Unless you have received this program directly from NYSOL pursuant
 * to the terms of a commercial license agreement with NYSOL, then
 * this program is licensed to you under the terms of the GNU Affero General
 * Public License (AGPL) as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
 * for more details.

 ////////// LICENSE INFO ////////////////////*/
// =============================================================================
/// kgmergeHelp.h : kgmerge help
// =============================================================================
_title="マッチング併合";
_doc="\
\n\
MMERGE マッチング併合\n\
\n\
指定した複数のファイルについて、指定した項目(「併合キー項目」と呼ぶ)の順序を保ちながら併合する。\n\
それぞれのファイルは指定した項目で事前に並べ替えておかなければならない。\n\
項目ごとに文字の昇順ソート,文字の降順ソート,数値の昇順ソート、数値の降順ソートが可能。\n\
";

