/* ////////// LICENSE INFO ////////////////////

 * Copyright (C) 2013 by NYSOL CORPORATION
 *
 * Unless you have received this program directly from NYSOL pursuant
 * to the terms of a commercial license agreement with NYSOL, then
 * this program is licensed to you under the terms of the GNU Affero General
 * Public License (AGPL) as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
 * for more details.

 ////////// LICENSE INFO ////////////////////*/
#pragma once
#include <kgcut.h>
#include <kgfifo.h>
#include <kg2tee.h>
#include <kgexcmd.h>
#include <kgsum.h>
#include <kgcal.h>
#include <kgcat.h>
#include <kgjoin.h>
#include <kgload.h>
#include <kg2cross.h>
#include <kgaccum.h>
#include <kgavg.h>
#include <kgbest.h>
#include <kgbucket.h>
#include <kgchgnum.h>
#include <kgchgstr.h>
#include <kgcombi.h>
#include <kgcommon.h>
#include <kgcount.h>
#include <kgcross.h>
#include <kgdelnull.h>
#include <kgdformat.h>
#include <kgduprec.h>
#include <kgfldname.h>
#include <kgfsort.h>
#include <kghashavg.h>
#include <kghashsum.h>
#include <kgkeybreak.h>
#include <kgmbucket.h>
#include <kgmvavg.h>
#include <kgmvsim.h>
#include <kgmvstats.h>
#include <kgnewnumber.h>
#include <kgnewrand.h>
#include <kgnewstr.h>
#include <kgnjoin.h>
#include <kgnormalize.h>
#include <kgnrcommon.h>
#include <kgnrjoin.h>
#include <kgnullto.h>
#include <kgnumber.h>
#include <kgpadding.h>
#include <kgpaste.h>
#include <kgproduct.h>
#include <kgrand.h>
#include <kgrjoin.h>
#include <kgsed.h>
#include <kgsep.h>
#include <kgshuffle.h>
#include <kgsel.h>
#include <kgselnum.h>
#include <kgselrand.h>
#include <kgselstr.h>
#include <kgsetstr.h>
#include <kgshare.h>
#include <kgsim.h>
#include <kgslide.h>
#include <kgsplit.h>
#include <kgstats.h>
#include <kgsummary.h>
#include <kgtonull.h>
#include <kgtra.h>
#include <kgtrafld.h>
#include <kgtraflg.h>
#include <kguniq.h>
#include <kgvcat.h>
#include <kgvcommon.h>
#include <kgvcount.h>
#include <kgvdelim.h>
#include <kgvdelnull.h>
#include <kgvjoin.h>
#include <kgvnullto.h>
#include <kgvreplace.h>
#include <kgvsort.h>
#include <kgvuniq.h>
#include <kgwindow.h>
#include <kgarff2csv.h>
#include <kgxml2csv.h>
#include <kgtab2csv.h>
#include <kgpyfunc.h>
#include <kgunicat.h>
#include <kg2cat.h>
#include <kgkmeans.h>
#include <kgsortchk.h>

